local traceTiles = require("traceTiles")
local debugWorldDraw = require("debugWorldDraw")

local rng = love.math.newRandomGenerator(os.time())

local world,terrainBody,playerBody,terrainGrid
local playerCanJump = 0 --number of fixtures touching feet sensor
local tileWidth,tileHeight = 32,32

local function newTileGrid(width,height,density)
	local grid = {}
	for x=0,width-1 do
		grid[x] = {}
		for y=0,height-1 do
			if (x==0 or y==0 or x>width-2 or y>height-2) then --grid border
				grid[x][y] = true
			elseif (rng:random()<density) then
				grid[x][y] = true
			end
		end
	end
	return grid
end

local function constructTerrain(body,grid,tileWidth,tileHeight)
	local fixtures = body:getFixtureList()
	for i=1,#fixtures do
		fixtures[i]:destroy()
	end
	for points in traceTiles(grid,tileWidth,tileHeight,true) do
		local shape = love.physics.newChainShape(true,unpack(points))
		love.physics.newFixture(body,shape,1)
	end
end

local function beginContact(fixture1,fixture2,contact)
	local userData1,userData2 = fixture1:getUserData(),fixture2:getUserData()
	if (userData1 == "feet" or userData2 == "feet") then
		playerCanJump = playerCanJump+1
	end
end

local function endContact(fixture1,fixture2,contact)
	local userData1,userData2 = fixture1:getUserData(),fixture2:getUserData()
	if (userData1 == "feet" or userData2 == "feet") then
		playerCanJump = playerCanJump-1
	end
end

function love.load()
	love.graphics.setBackgroundColor(128,128,128,255)

	world = love.physics.newWorld(0,1000,true)
	world:setCallbacks(beginContact,endContact)
	
	local px,py = 4,90 --starting player position
	
	terrainBody = love.physics.newBody(world,0,0,"static")
	terrainGrid = newTileGrid(100,100,0.2)
	for x=px-2,px+2 do --clear starting player position
		for y=py-2,py+2 do
			terrainGrid[x][y] = nil
		end
	end

	constructTerrain(terrainBody,terrainGrid,tileWidth,tileHeight)
	
	playerBody = love.physics.newBody(world,tileWidth*px,tileHeight*py,"dynamic")
	playerBody:setBullet(true)
	playerBody:setFixedRotation(true)
	
	local torsoShape = love.physics.newRectangleShape(tileWidth*0.75,tileHeight*0.75)
	local torsoFixture = love.physics.newFixture(playerBody,torsoShape)
	torsoFixture:setFriction(0.75)
	
	local feetShape = love.physics.newRectangleShape(0,(tileHeight*0.75-8)/2+4,tileWidth*0.75-2,8,0)
	local feetFixture = love.physics.newFixture(playerBody,feetShape)
	feetFixture:setSensor(true)
	feetFixture:setUserData("feet")
end

function love.update(dt)
	local vx,vy = playerBody:getLinearVelocity()
	local m = playerBody:getMass()
	if (vx < 250 and love.keyboard.isDown("right")) then
		playerBody:applyForce(m*1000,0)
	elseif (vx > -250 and love.keyboard.isDown("left")) then
		playerBody:applyForce(m*-1000,0)
	end
	
	world:update(dt)
end

function love.draw()
	local x,y = playerBody:getWorldCenter()
	local width,height = love.graphics.getDimensions()
	love.graphics.push()
	love.graphics.translate(-x+width/2,-y+height/2) --center on player body
	debugWorldDraw(world,x-width/2,y-height/2,width,height) --area around player body
	love.graphics.pop()
end

function love.keypressed(key,isrepeat)
	if (playerCanJump>0 and (key == " " or key == "up")) then
		playerBody:applyLinearImpulse(0,playerBody:getMass()*-600)
	end
end
