local coroutine_wrap,coroutine_yield = coroutine.wrap,coroutine.yield

local function get(t,x,y) --2d table get
	if (t[x]) then return t[x][y] end
end

local function set(t,x,y,v) --2d table set
	if (t[x]) then t[x][y] = v
	else t[x] = {[y]=v} end
end

local function addVertice(vertices,w,h,x1,y1,x2,y2)
	x1,y1,x2,y2 = x1*w,y1*h,x2*w,y2*h
	local point = get(vertices,x1,y1)
	if (point) then --2 vertices share the same start point
		point[3],point[4] = x2,y2
	else
		set(vertices,x1,y1,{x2,y2})
	end
end

local function tracetileGrid(tileGrid,tileWidth,tileHeight,chainshapeFriendly)
	local w,h = tileWidth,tileHeight
	local vertices = {}
	
	for x,row in pairs(tileGrid) do
		for y,value in pairs(row) do
			if (not get(tileGrid,x-1,y)) then --left
				addVertice(vertices,w,h,x,y+1,x,y) --bottom to top
			end
			if (not get(tileGrid,x+1,y)) then --right
				addVertice(vertices,w,h,x+1,y,x+1,y+1) --top to bottom
			end
			if (not get(tileGrid,x,y-1)) then --top
				addVertice(vertices,w,h,x,y,x+1,y) --left to right
			end
			if (not get(tileGrid,x,y+1)) then --bottom
				addVertice(vertices,w,h,x+1,y+1,x,y+1) --right to left
			end
		end
	end
	
	for sx,row in pairs(vertices) do
		for sy in pairs(row) do
			while (get(vertices,sx,sy)) do --repeat until there are no more vertices starting at this point
				local point = get(vertices,sx,sy) --prepare starting vertex
				local x1,y1 = sx,sy
				local x2,y2
				if (point[3]) then --overlap
					x2,y2 = point[3],point[4]
					point[3],point[4] = nil,nil
				else
					x2,y2 = point[1],point[2]
					set(vertices,x1,y1,nil) --vertex done
				end
				
				local points = {x1,y1,x2,y2}
				local pointsCount = 4
				
				while (get(vertices,x2,y2)) do --repeat until we passed all vertices in this polyline
					local nextPoint = get(vertices,x2,y2)
					if (nextPoint[3]) then --there are 2 vertices starting from this point, decide where to go
						local xa,ya,xb,yb = nextPoint[1],nextPoint[2],nextPoint[3],nextPoint[4]
						nextPoint[3],nextPoint[4] = nil,nil
						if ((x1 < x2) and (ya > yb))
						or ((x1 > x2) and (ya < yb))
						or ((y1 < y2) and (xa < xb))
						or ((y1 > y2) and (xa > xb)) then --maintain clockwise direction
							x1,y1 = x2,y2
							x2,y2 = xa,ya
							nextPoint[1],nextPoint[2] = xb,yb
						else
							x1,y1 = x2,y2
							x2,y2 = xb,yb
						end
					else
						set(vertices,x2,y2,nil) --vertex done
						x1,y1 = x2,y2
						x2,y2 = nextPoint[1],nextPoint[2]
					end
					
					local px,py = points[pointsCount-3],points[pointsCount-2]
					
					--this groups neighbouring vertices into a single line
					if (px==x2) then --same direction as previous, overwrite
						points[pointsCount] = y2
					elseif (py==y2) then --same direction as previous, overwrite
						points[pointsCount-1] = x2
					else --different direction
						points[pointsCount+1],points[pointsCount+2] = x2,y2
						pointsCount = pointsCount + 2
					end
				end
				
				local xa,ya,xb,yb = points[3],points[4],points[pointsCount-3],points[pointsCount-2]
				
				--this happens when the start point is inside what should be a grouped line
				if (xa==xb or ya==yb) then --remove redundant start point
					points[1],points[2] = xb,yb
					points[pointsCount-1],points[pointsCount] = nil,nil
					pointsCount = pointsCount - 2
				end
				
				if (chainshapeFriendly) then --remove last point since chainshape wraps around
					points[pointsCount-1],points[pointsCount] = nil,nil
				end
				
				coroutine_yield(points)
			end
		end
	end
end

local function iterator(tileGrid,tileWidth,tileHeight,chainshapeFriendly)
	return coroutine_wrap(function() tracetileGrid(tileGrid,tileWidth,tileHeight,chainshapeFriendly) end)
end

return iterator